@@ -23,32 +23,36 @@ from werkzeug.exceptions import Unauthorized, Forbidden
 class MyResourceProtector(ResourceProtector):
     def __init__(self, logger, require_token=True):
         super().__init__()
-        self.require_token = require_token
         self.logger = logger
+        self.require_token = require_token
 
     def check_permission(self, permission: str) -> bool:
+        return self.check_permissions([permission])
+
+    def check_permissions(self, permissions: list) -> bool:
         try:
-            self.acquire_token(permission)
+            self.acquire_token(permissions)
             return True
         except Exception as error:
             self.logger.error(f"Acquiring token failed {error}")
             return False
 
-    def acquire_token(self, permissions=None):
-        """A method to acquire current valid token with the given scope.
+    def is_super_admin(self):
+        if token := self.acquire_token():
+            return token.is_super_admin()
+        return False
 
-        :param permissions: a list of required permissions
-        :return: token object
-        """
+    def get_token_permissions(self, role_permission_mapping):
+        if token := self.acquire_token():
+            return token.get_token_permissions(role_permission_mapping)
+        return []
+
+    def acquire_token(self, permissions=None):
         request = HttpRequest(_req.method, _req.full_path, _req.data, _req.headers)
         request.req = _req
-        # backward compatible
-        if isinstance(permissions, str):
-            permissions = [permissions]
+        token = ""
         if self.require_token:
             token = self.validate_request(permissions, request)
-        else:
-            token = ""
         token_authenticated.send(self, token=token)
         ctx = _app_ctx_stack.top
         ctx.authlib_server_oauth2_token = token
@@ -82,7 +86,6 @@ class MyResourceProtector(ResourceProtector):
         return wrapper
 
     def validate_request(self, permissions, request):
-        """Validate the request and return a token."""
         validator, token_string = self.parse_request_authorization(request)
         validator.validate_request(request)
         token = validator.authenticate_token(token_string)
@@ -91,6 +94,29 @@ class MyResourceProtector(ResourceProtector):
 
 
 class JWT(JWTBearerToken):
+    def __get_roles(self):
+        if any(x not in self for x in ["azp", "resource_access"]):
+            return []
+        if self["azp"] not in self["resource_access"]:
+            return []
+        if "roles" not in self["resource_access"][self["azp"]]:
+            return []
+        return self["resource_access"][self["azp"]]["roles"]
+
+    def get_token_permissions(self, role_permission_mapping):
+        permissions = []
+        roles = self.__get_roles()
+        if not role_permission_mapping or not roles:
+            return permissions
+        if self.is_super_admin():
+            for role in role_permission_mapping:
+                permissions.extend(role_permission_mapping[role])
+            return permissions
+        for role in roles:
+            if role in role_permission_mapping:
+                permissions.extend(role_permission_mapping[role])
+        return permissions
+
     def has_permissions(
         self,
         permissions,
@@ -99,25 +125,18 @@ class JWT(JWTBearerToken):
     ):
         if not permissions:
             return True
-        if any(x not in self for x in ["azp", "resource_access"]):
-            return False
-        if self["azp"] not in self["resource_access"]:
-            return False
-        resource_access = self["resource_access"][self["azp"]]
-        if "roles" not in resource_access:
-            return False
-        if super_admin_role in resource_access["roles"]:
+        if self.is_super_admin(super_admin_role):
             return True
         if not role_permission_mapping:
             return False
-        user_permissions = []
-        for role in resource_access["roles"]:
-            if role in role_permission_mapping:
-                user_permissions.extend([x for x in role_permission_mapping[role]])
+        user_permissions = self.get_token_permissions(role_permission_mapping)
         if any(x in user_permissions for x in permissions):
             return True
         return False
 
+    def is_super_admin(self, super_admin_role="role_super_admin"):
+        return super_admin_role in self.__get_roles()
+
 
 class JWTValidator(BearerTokenValidator, ABC):
     TOKEN_TYPE = "bearer"
@@ -140,14 +159,12 @@ class JWTValidator(BearerTokenValidator, ABC):
         self.static_issuer = static_issuer
         self.static_public_key = static_public_key
         self.logger = logger
-        self.public_key = ""
         self.realms = realms if realms else []
-        claims_options = {
+        self.claims_options = {
             "exp": {"essential": True},
             "azp": {"essential": True},
             "sub": {"essential": True},
         }
-        self.claims_options = claims_options
         self.role_permission_mapping = None
         self.super_admin_role = super_admin_role
         self.remote_token_validation = remote_token_validation
@@ -172,16 +189,13 @@ class JWTValidator(BearerTokenValidator, ABC):
         if not issuer:
             return None
         realm_config = self.__get_realm_config_by_issuer(issuer)
+        public_key = ""
         if "public_key" in realm_config:
-            self.public_key = (
-                "-----BEGIN PUBLIC KEY-----\n"
-                + realm_config["public_key"]
-                + "\n-----END PUBLIC KEY-----"
-            )
+            public_key = f'-----BEGIN PUBLIC KEY-----\n{realm_config["public_key"]}\n-----END PUBLIC KEY-----'
         try:
             claims = jwt.decode(
                 token_string,
-                self.public_key,
+                public_key,
                 claims_options=self.claims_options,
                 claims_cls=self.token_cls,
             )
@@ -217,7 +231,6 @@ class JWTValidator(BearerTokenValidator, ABC):
         return self.realm_config_cache[issuer]
 
     def validate_token(self, token, permissions, request):
-        """Check if token is active and matches the requested permissions."""
         super().validate_token(token, None, request)
         if not token.has_permissions(
             permissions, self.role_permission_mapping, self.super_admin_role
@@ -238,15 +251,6 @@ class JWTValidator(BearerTokenValidator, ABC):
 
 
 class InsufficientPermissionError(OAuth2Error):
-    """The request requires higher privileges than provided by the
-    access token. The resource server SHOULD respond with the HTTP
-    403 (Forbidden) status code and MAY include the "scope"
-    attribute with the scope necessary to access the protected
-    resource.
-
-    https://tools.ietf.org/html/rfc6750#section-3.1
-    """
-
     error = "insufficient_permission"
     description = (
         "The request requires higher privileges than provided by the access token."
